@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Solaris interface driver for dhcpcd
- * Copyright (c) 2016-2023 Roy Marples <roy@marples.name>
+ * Copyright (c) 2016-2025 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -154,7 +154,7 @@ if_opensockets_os(struct dhcpcd_ctx *ctx)
 	 * We will fail noisily elsewhere anyway. */
 #endif
 
-	ctx->link_fd = socket(PF_ROUTE,
+	ctx->link_fd = xsocket(PF_ROUTE,
 	    SOCK_RAW | SOCK_CLOEXEC | SOCK_NONBLOCK, 0);
 
 	if (ctx->link_fd == -1) {
@@ -180,12 +180,13 @@ if_closesockets_os(struct dhcpcd_ctx *ctx)
 	struct priv		*priv;
 
 	priv = (struct priv *)ctx->priv;
-	if (priv->pf_inet6_fd != -1)
+	if (priv && priv->pf_inet6_fd != -1)
 		close(priv->pf_inet6_fd);
 #endif
 
 	/* each interface should have closed itself */
 	free(ctx->priv);
+	ctx->priv = NULL;
 }
 
 int
@@ -204,9 +205,6 @@ if_carrier(struct interface *ifp, __unused const void *ifadata)
 	kstat_named_t		*knp;
 	link_state_t		linkstate;
 
-	if (if_getflags(ifp) == -1)
-		return LINK_UNKNOWN;
-
 	kcp = kstat_open();
 	if (kcp == NULL)
 		goto err;
@@ -225,7 +223,7 @@ if_carrier(struct interface *ifp, __unused const void *ifadata)
 
 	switch (linkstate) {
 	case LINK_STATE_UP:
-		ifp->flags |= IFF_UP;
+		ifp->flags |= IFF_UP; /* XXX Why? */
 		return LINK_UP;
 	case LINK_STATE_DOWN:
 		return LINK_DOWN;
@@ -603,8 +601,8 @@ if_route0(struct dhcpcd_ctx *ctx, struct rtm *rtmsg,
 		else if (!gateway_unspec)
 			rtm->rtm_flags |= RTF_GATEWAY;
 
-		if (rt->rt_dflags & RTDF_STATIC)
-			rtm->rtm_flags |= RTF_STATIC;
+		/* Make static so that in.routed does not delete it */
+		rtm->rtm_flags |= RTF_STATIC;
 
 		if (rt->rt_mtu != 0) {
 			rtm->rtm_inits |= RTV_MTU;
@@ -727,7 +725,7 @@ if_route_get(struct dhcpcd_ctx *ctx, struct rt *rt)
 
 	if_route0(ctx, &rtm, RTM_GET, rt);
 	rt = NULL;
-	s = socket(PF_ROUTE, SOCK_RAW | SOCK_CLOEXEC, 0);
+	s = xsocket(PF_ROUTE, SOCK_RAW | SOCK_CLOEXEC, 0);
 	if (s == -1)
 		return NULL;
 	if (write(s, &rtm, rtm.hdr.rtm_msglen) == -1)
@@ -1061,6 +1059,7 @@ if_ifinfo(struct dhcpcd_ctx *ctx, const struct if_msghdr *ifm)
 		state = LINK_UP;
 		flags |= IFF_UP;
 	}
+	ifp->mtu = if_mtu(ifp);
 	dhcpcd_handlecarrier(ifp, state, flags);
 	return 0;
 }
@@ -1745,11 +1744,4 @@ if_setup_inet6(__unused const struct interface *ifp)
 {
 
 }
-
-int
-ip6_forwarding(__unused const char *ifname)
-{
-
-	return 1;
-}
 #endif
